from mutagen.flac import FLAC, Picture
from mutagen.id3 import PictureType
from deemix.types.Track import Track


# Adds tags to a FLAC file
def tagFLAC(path, track: Track, save: dict):
    # Delete exsisting tags
    tag = FLAC(path)
    tag.delete()
    tag.clear_pictures()

    # List of metadata: https://picard-docs.musicbrainz.org/en/appendices/tag_mapping.html

    # For MusicBrainz
    tag["MEDIA"] = "Digital Media"

    if save["title"]:
        tag["TITLE"] = track.title

    if save["album"]:
        tag["ALBUM"] = track.album.title

    # multi artists
    # > ARTIST can be reliably split, the standards (id3, Vorbis, mp4) explicitly tell us how:
    # > null-separated strings (id3v2.4) or multiple ARTIST frames (Vorbis, mp4).
    # from https://github.com/navidrome/navidrome/issues/238#issuecomment-1613474074
    if save["artist"] and track.artists:
        if save["multiArtistSeparator"] == "default":
            # default for FLAC means multiple ARTIST tag, thanks for mutagen
            # ARTIST=King Gizzard & The Lizard Wizard
            # ARTIST=Mild High Club
            tag["ARTIST"] = track.artists
        else:
            if save["multiArtistSeparator"] == "nothing":
                tag["ARTIST"] = track.mainArtist.name
            else:
                tag["ARTIST"] = track.artistsString

    # Tag ARTISTS is added to keep the multiartist support when using a non standard tagging method
    # https://picard-docs.musicbrainz.org/en/technical/tag_mapping.html#artists
    # The null-separated string is for mp3, but beets support it: https://github.com/beetbox/beets/pull/4743
    # Thus I set it regardless of save["artist"] and save["multiArtistSeparator"]
    if save["artists"] and len(track.artists) > 1:
        tag["ARTISTS"] = "\0".join(track.artists)

    # TODO multi artists
    if save["albumArtist"] and len(track.album.artists):
        if save["singleAlbumArtist"] and track.album.mainArtist.save:
            tag["ALBUMARTIST"] = track.album.mainArtist.name
        else:
            # ALBUMARTIST=King Gizzard & The Lizard Wizard
            # ALBUMARTIST=Mild High Club
            tag["ALBUMARTIST"] = track.album.artists

    if save["trackNumber"]:
        tag["TRACKNUMBER"] = str(track.trackNumber)
    if save["trackTotal"]:
        tag["TRACKTOTAL"] = str(track.album.trackTotal)
    if save["discNumber"]:
        tag["DISCNUMBER"] = str(track.discNumber)
    if save["discTotal"]:
        tag["DISCTOTAL"] = str(track.album.discTotal)
    if save["genre"]:
        tag["GENRE"] = track.album.genre

    # YEAR tag is not suggested as a standard tag
    # Being YEAR already contained in DATE will only use DATE instead
    # Reference: https://www.xiph.org/vorbis/doc/v-comment.html#fieldnames
    if save["date"]:
        tag["DATE"] = track.dateString
    elif save["year"]:
        tag["DATE"] = str(track.date.year)

    if save["length"]:
        tag["LENGTH"] = str(int(track.duration) * 1000)
    if save["bpm"] and track.bpm:
        tag["BPM"] = str(track.bpm)
    if save["label"]:
        tag["PUBLISHER"] = save["labelMap"].get(label := track.album.label, label)
    if save["isrc"]:
        tag["ISRC"] = track.ISRC
    if save["barcode"]:
        tag["BARCODE"] = track.album.barcode
    if save["explicit"]:
        tag["ITUNESADVISORY"] = "1" if track.explicit else "0"
    if save["replayGain"]:
        tag["REPLAYGAIN_TRACK_GAIN"] = track.replayGain
    if track.lyrics.unsync and save["unsyncedLyrics"]:
        # This is not standard, but widely accepted
        tag["UNSYNCEDLYRICS"] = track.lyrics.unsync
    if track.lyrics.sync and save["syncedLyrics"]:
        tag["LYRICS"] = track.lyrics.sync

    # TODO add them?
    for role, contributors in track.contributors.items():
        # print(f'{role} is {contributors}')
        # main_artist is ['Taylor Swift']
        # artist is ['Taylor Swift', 'Derek Garten', 'Dan Burns', 'Matt Billingslea', 'Amos Heller', 'Max Bernstein', 'Mike Meadows']
        # composer is ['Taylor Swift', 'Max Martin', 'Shellback', 'Ali Payami']
        if role == "composer" and save["composer"]:
            # mutagen will create multiple COMPOSER tag if len(contributors) > 1
            tag[role.upper()] = contributors
        elif save["involvedPeople"]:
            if role in ["author", "engineer", "mixer", "producer", "writer"]:
                tag[role.upper()] = track.contributors[role]
            elif role == "musicpublisher":
                tag["ORGANIZATION"] = track.contributors["musicpublisher"]

    if save["copyright"] and track.copyright:
        tag["COPYRIGHT"] = track.copyright
    if (
        save["savePlaylistAsCompilation"]
        and track.playlist
        or track.album.recordType == "compile"
    ):
        tag["COMPILATION"] = "1"

    if save["source"]:
        tag["SOURCE"] = "Deezer"
        # Access the track using https://www.deezer.com/track/$SOURCEID
        tag["SOURCEID"] = str(track.id)

    if save["rating"]:
        # MusicBrainz rating is a value between 1 and 5, https://musicbrainz.org/doc/Rating_System
        # The biggest rank in deezer I've seen is 998513.
        # I guess that's the popularity of the whole world, and max is 100_0000.
        rank = round((int(track.rank) / 10000))
        tag["RATING"] = str(rank)

    if save["cover"] and track.album.embeddedCoverPath:
        image = Picture()
        image.type = PictureType.COVER_FRONT
        image.mime = (
            "image/png"
            if str(track.album.embeddedCoverPath).endswith("png")
            else "image/jpeg"
        )
        with open(track.album.embeddedCoverPath, "rb") as f:
            image.data = f.read()
        tag.add_picture(image)

    tag.save(deleteid3=True)
